/*!
 * @file  RTC_Joystick.cpp
 * @brief ゲームパッド・ジョイスティックのRTC
 *
 * @author Masaharu Shimizu@Future Robotics Technology Center, Chiba Institute of Technology
 *
 * @date 2009/08/19 デバイスをコンフィグで選択可能にする。
 * @date 2009/08/24 Joy2Velの機能を取り込む 取り合えずIIS.idl形式の速度を出力
 * @date 2009/10/26 速度出力がジョイスティックの変化があったときのみ出力される方法を常に出力する方法に変更
 * @date 2010/07/06 OpenRTM1.0対応
 */

#include "RTC_Joystick.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_joystick_spec[] =
  {
    "implementation_id", "RTC_Joystick",
    "type_name",         "RTC_Joystick",
    "description",       "Joystick",
    "version",           "1.0.0",
    "vendor",            "Masaharu Shimizu@Chiba Institute of Technology",
    "category",          "Device",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.vxLimit", "0.3",
    "conf.default.vyLimit", "0.3",
    "conf.default.wLimit", "0.3",
    "conf.default.joystick_dev_name", "/dev/input/js0",
    ""
  };
// </rtc-template>

RTC_Joystick::RTC_Joystick(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_outButtonOut("outButton", m_outButton),
    m_outAxisOut("outAxis", m_outAxis),
    m_outVelIISOut("outVelIIS", m_outVelIIS)

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers

  // Set OutPort buffer
  registerOutPort("outButton", m_outButtonOut);
  registerOutPort("outAxis", m_outAxisOut);
  registerOutPort("outVelIIS", m_outVelIISOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

}

RTC_Joystick::~RTC_Joystick()
{
}


RTC::ReturnCode_t RTC_Joystick::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("vxLimit", m_vxLimit, "0.3");
  bindParameter("vyLimit", m_vyLimit, "0.3");
  bindParameter("wLimit", m_wLimit, "0.3");
  bindParameter("joystick_dev_name", m_joystick_dev_name, "/dev/input/js0");

  // </rtc-template>
  return RTC::RTC_OK;
}



RTC::ReturnCode_t RTC_Joystick::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Joystick::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t RTC_Joystick::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t RTC_Joystick::onActivated(RTC::UniqueId ec_id)
{
	char *axis_names[ABS_MAX + 1] = {
			"X", "Y", "Z", "Rx", "Ry", "Rz", "Throttle", "Rudder",
			"Wheel", "Gas", "Brake", "?", "?", "?", "?", "?",
			"Hat0X", "Hat0Y", "Hat1X", "Hat1Y", "Hat2X", "Hat2Y", "Hat3X", "Hat3Y",
			"?", "?", "?", "?", "?", "?", "?",
	};

	char *button_names[KEY_MAX - BTN_MISC + 1] = {
			"Btn0", "Btn1", "Btn2", "Btn3", "Btn4", "Btn5", "Btn6", "Btn7", "Btn8", "Btn9", "?", "?", "?", "?", "?", "?",
			"LeftBtn", "RightBtn", "MiddleBtn", "SideBtn", "ExtraBtn", "ForwardBtn", "BackBtn", "TaskBtn", "?", "?", "?", "?", "?", "?", "?", "?",
			"Trigger", "ThumbBtn", "ThumbBtn2", "TopBtn", "TopBtn2", "PinkieBtn", "BaseBtn", "BaseBtn2", "BaseBtn3", "BaseBtn4", "BaseBtn5", "BaseBtn6", "BtnDead",
			"BtnA", "BtnB", "BtnC", "BtnX", "BtnY", "BtnZ", "BtnTL", "BtnTR", "BtnTL2", "BtnTR2", "BtnSelect", "BtnStart", "BtnMode", "BtnThumbL", "BtnThumbR", "?",
			"?", "?", "?", "?", "?", "?", "?", "?", "?", "?", "?", "?", "?", "?", "?", "?",
			"WheelBtn", "Gear up",
	};

	int i;
	axes = 2;
	buttons = 2;
	version = 0x000800;
	strcpy(name, "Unknown");

	if ((fd = open(m_joystick_dev_name, O_RDONLY)) < 0) {
		perror("RTC_Joystick::onInitialize");
		return RTC::RTC_ERROR;
	}

	fcntl(fd, F_SETFL, O_NONBLOCK);

	ioctl(fd, JSIOCGVERSION, &version);
	ioctl(fd, JSIOCGAXES, &axes);
	ioctl(fd, JSIOCGBUTTONS, &buttons);
	ioctl(fd, JSIOCGNAME(NAME_LENGTH), name);
	ioctl(fd, JSIOCGAXMAP, axmap);
	ioctl(fd, JSIOCGBTNMAP, btnmap);



	printf("Driver version is %d.%d.%d.\n",
			version >> 16, (version >> 8) & 0xff, version & 0xff);
	if (buttons > 0 && btnmap[0] < BTN_MISC) {
		/* btnmap out of range for names. Don't print any. */
		printf("Joystick (%s) has %d axes ", name, axes);
		printf("and %d buttons.\n", buttons);
	} else {
		printf("Joystick (%s) has %d axes (", name, axes);
		for (i = 0; i < axes; i++)
			printf("%s%s", i > 0 ? ", " : "", axis_names[axmap[i]]);
		puts(")");

		printf("and %d buttons (", buttons);
		for (i = 0; i < buttons; i++)
			printf("%s%s", i > 0 ? ", " : "", button_names[btnmap[i] - BTN_MISC]);
		puts(").");
	}

	axis = (int*)calloc(axes, sizeof(int));
	button = (char*)calloc(buttons, sizeof(char));

	m_vxGain = m_vxLimit/32767.0;
	m_vyGain = m_vyLimit/(-32767.0);
	m_wGain = m_wLimit/(-32767.0);

	return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Joystick::onDeactivated(RTC::UniqueId ec_id)
{
	close(fd);
	free(axis);
	free(button);
	return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Joystick::onExecute(RTC::UniqueId ec_id)
{
	int i;
	if (read(fd, &js, sizeof(struct js_event)) == sizeof(struct js_event)) {
		switch(js.type & ~JS_EVENT_INIT) {
		case JS_EVENT_BUTTON:
			button[js.number] = js.value;

			m_outButton.data.length(buttons);
			for(i=0;i<buttons;i++){
				m_outButton.data[i] = button[i];
			}
			m_outButtonOut.write();
			break;

		case JS_EVENT_AXIS:
			axis[js.number] = js.value;

			m_outAxis.data.length(axes);
			for(i=0;i<axes;i++){
				m_outAxis.data[i] = axis[i];
			}
			m_outAxisOut.write();
			break;
		}

		printf("\r");

		if (axes) {
			printf("Axes: ");
			for (i = 0; i < axes; i++)
				printf("%2d:%6d ", i, axis[i]);
		}

		if (buttons) {
			printf("Buttons: ");
			for (i = 0; i < buttons; i++)
				printf("%2d:%s ", i, button[i] ? "on " : "off");
		}

		fflush(stdout);

	}


	//IIS.idlインターフェイス　vxがロボット進行方向
	m_outVelIIS.vx = axis[1] * -1 * m_vxGain;
	m_outVelIIS.vy = axis[0] * m_vyGain;
	m_outVelIIS.w = axis[2] * m_wGain;
	m_outVelIISOut.write();

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Joystick::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t RTC_Joystick::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t RTC_Joystick::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t RTC_Joystick::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t RTC_Joystick::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{

  void RTC_JoystickInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_joystick_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_Joystick>,
                             RTC::Delete<RTC_Joystick>);
  }

};



