/*!
 * @file  RTC_Joystick.h
 * @brief ゲームパッド・ジョイスティックのRTC
 *
 * @author Masaharu Shimizu@Future Robotics Technology Center, Chiba Institute of Technology
 *
 * @date 2009/08/19 デバイスをコンフィグで選択可能にする。
 * @date 2009/08/24 Joy2Velの機能を取り込む 取り合えずIIS.idl形式の速度を出力
 * @date 2009/10/28 OpenRTM1.0RC対応
 * @date 2010/07/06 OpenRTM1.0対応
 */

#ifndef RTC_JOYSTICK_H
#define RTC_JOYSTICK_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "IISStub.h"
// </rtc-template>

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>

#include <linux/input.h>
#include <linux/joystick.h>

#define NAME_LENGTH 128

#define JOY2VX_NUM 1
#define JOY2VY_NUM 2
#define JOY2W_NUM 4

using namespace RTC;

class RTC_Joystick  : public RTC::DataFlowComponentBase
{
 public:
  RTC_Joystick(RTC::Manager* manager);
  ~RTC_Joystick();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry()
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  double m_vxLimit;
  double m_vyLimit;
  double m_wLimit;
  char m_joystick_dev_name[30];

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedBooleanSeq m_outButton;
  OutPort<TimedBooleanSeq> m_outButtonOut;
  TimedLongSeq m_outAxis;
  OutPort<TimedLongSeq> m_outAxisOut;
  IIS::TimedVelocity m_outVelIIS;
  OutPort<IIS::TimedVelocity> m_outVelIISOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:
	 double m_vxGain;
	 double m_vyGain;
	 double m_wGain;

	 int fd;
	 unsigned char axes;
	 unsigned char buttons;
	 int version;
	 char name[NAME_LENGTH];
	 uint16_t btnmap[KEY_MAX - BTN_MISC + 1];
	 uint8_t axmap[ABS_MAX + 1];
	 struct js_event js;
	 int *axis;
	 char *button;
	 double vx,vy,w;

};


extern "C"
{
  DLL_EXPORT void RTC_JoystickInit(RTC::Manager* manager);
};

#endif // RTC_JOYSTICK_H

